]> git.xonotic.org Git - xonotic/darkplaces.git/blobdiff - csprogs.c
Convert \ to / when loading texture from Q3 shader
[xonotic/darkplaces.git] / csprogs.c
index b1598ddadd97a68df5172545aaec41f2dd251e63..fa1673b4c375254ab44b185e838041e289ad4505 100644 (file)
--- a/csprogs.c
+++ b/csprogs.c
@@ -20,10 +20,10 @@ void CL_VM_PreventInformationLeaks(void)
        prvm_prog_t *prog = CLVM_prog;
        if(!cl.csqc_loaded)
                return;
-       CSQC_BEGIN
-               VM_ClearTraceGlobals(prog);
-               PRVM_clientglobalfloat(trace_networkentity) = 0;
-       CSQC_END
+CSQC_BEGIN
+       VM_ClearTraceGlobals(prog);
+       PRVM_clientglobalfloat(trace_networkentity) = 0;
+CSQC_END
 }
 
 //[515]: these are required funcs
@@ -227,7 +227,7 @@ static void CSQC_SetGlobals (double frametime)
        prvm_prog_t *prog = CLVM_prog;
        CSQC_BEGIN
                PRVM_clientglobalfloat(time) = cl.time;
-               PRVM_clientglobalfloat(cltime) = realtime; // Spike named it that way.
+               PRVM_clientglobalfloat(cltime) = host.realtime; // Spike named it that way.
                PRVM_clientglobalfloat(frametime) = frametime;
                PRVM_clientglobalfloat(servercommandframe) = cls.servermovesequence;
                PRVM_clientglobalfloat(clientcommandframe) = cl.movecmd[0].sequence;
@@ -238,7 +238,7 @@ static void CSQC_SetGlobals (double frametime)
                VectorCopy(cl.csqc_vieworiginfromengine, cl.csqc_vieworigin);
                VectorCopy(cl.csqc_viewanglesfromengine, cl.csqc_viewangles);
 
-               // LordHavoc: Spike says not to do this, but without pmove_org the
+               // LadyHavoc: Spike says not to do this, but without pmove_org the
                // CSQC is useless as it can't alter the view origin without
                // completely replacing it
                Matrix4x4_OriginFromMatrix(&cl.entities[cl.viewentity].render.matrix, pmove_org);
@@ -286,7 +286,6 @@ void CSQC_Think (prvm_edict_t *ed)
 }
 
 extern cvar_t cl_noplayershadow;
-extern cvar_t r_equalize_entities_fullbright;
 qboolean CSQC_AddRenderEdict(prvm_edict_t *ed, int edictnum)
 {
        prvm_prog_t *prog = CLVM_prog;
@@ -295,6 +294,7 @@ qboolean CSQC_AddRenderEdict(prvm_edict_t *ed, int edictnum)
        float scale;
        entity_render_t *entrender;
        dp_model_t *model;
+       prvm_vec3_t modellight_origin;
 
        model = CL_GetModelFromEdict(ed);
        if (!model)
@@ -346,8 +346,10 @@ qboolean CSQC_AddRenderEdict(prvm_edict_t *ed, int edictnum)
        if (!VectorLength2(entrender->glowmod))
                VectorSet(entrender->glowmod, 1, 1, 1);
 
-       // LordHavoc: use the CL_GetTagMatrix function on self to ensure consistent behavior (duplicate code would be bad)
-       CL_GetTagMatrix(prog, &entrender->matrix, ed, 0);
+       // LadyHavoc: use the CL_GetTagMatrix function on self to ensure consistent behavior (duplicate code would be bad)
+       // this also sets the custommodellight_origin for us
+       CL_GetTagMatrix(prog, &entrender->matrix, ed, 0, modellight_origin);
+       VectorCopy(modellight_origin, entrender->custommodellight_origin);
 
        // set up the animation data
        VM_GenerateFrameGroupBlend(prog, ed->priv.server->framegroupblend, ed);
@@ -363,9 +365,9 @@ qboolean CSQC_AddRenderEdict(prvm_edict_t *ed, int edictnum)
        // model light
        if (renderflags & RF_MODELLIGHT)
        {
-               if (PRVM_clientedictvector(ed, modellight_ambient)) VectorCopy(PRVM_clientedictvector(ed, modellight_ambient), entrender->modellight_ambient); else VectorClear(entrender->modellight_ambient);
-               if (PRVM_clientedictvector(ed, modellight_diffuse)) VectorCopy(PRVM_clientedictvector(ed, modellight_diffuse), entrender->modellight_diffuse); else VectorClear(entrender->modellight_diffuse);
-               if (PRVM_clientedictvector(ed, modellight_dir))     VectorCopy(PRVM_clientedictvector(ed, modellight_dir), entrender->modellight_lightdir);    else VectorClear(entrender->modellight_lightdir);
+               if (PRVM_clientedictvector(ed, modellight_ambient)) VectorCopy(PRVM_clientedictvector(ed, modellight_ambient), entrender->custommodellight_ambient); else VectorClear(entrender->custommodellight_ambient);
+               if (PRVM_clientedictvector(ed, modellight_diffuse)) VectorCopy(PRVM_clientedictvector(ed, modellight_diffuse), entrender->custommodellight_diffuse); else VectorClear(entrender->custommodellight_diffuse);
+               if (PRVM_clientedictvector(ed, modellight_dir))     VectorCopy(PRVM_clientedictvector(ed, modellight_dir), entrender->custommodellight_lightdir);    else VectorClear(entrender->custommodellight_lightdir);
                entrender->flags |= RENDER_CUSTOMIZEDMODELLIGHT;
        }
 
@@ -393,8 +395,6 @@ qboolean CSQC_AddRenderEdict(prvm_edict_t *ed, int edictnum)
        {
                if (!(entrender->effects & EF_FULLBRIGHT) && !(renderflags & RF_FULLBRIGHT))
                        entrender->flags |= RENDER_LIGHT;
-               else if(r_equalize_entities_fullbright.integer)
-                       entrender->flags |= RENDER_LIGHT | RENDER_EQUALIZE;
        }
        // hide player shadow during intermission or nehahra movie
        if (!(entrender->effects & (EF_NOSHADOW | EF_ADDITIVE | EF_NODEPTHTEST))
@@ -442,20 +442,20 @@ qboolean CL_VM_InputEvent (int eventtype, float x, float y)
        if(!cl.csqc_loaded)
                return false;
 
-       CSQC_BEGIN
-               if (!PRVM_clientfunction(CSQC_InputEvent))
-                       r = false;
-               else
-               {
-                       PRVM_clientglobalfloat(time) = cl.time;
-                       PRVM_clientglobaledict(self) = cl.csqc_server2csqcentitynumber[cl.playerentity];
-                       PRVM_G_FLOAT(OFS_PARM0) = eventtype;
-                       PRVM_G_FLOAT(OFS_PARM1) = x; // key or x
-                       PRVM_G_FLOAT(OFS_PARM2) = y; // ascii or y
-                       prog->ExecuteProgram(prog, PRVM_clientfunction(CSQC_InputEvent), "QC function CSQC_InputEvent is missing");
-                       r = CSQC_RETURNVAL != 0;
-               }
-       CSQC_END
+CSQC_BEGIN
+       if (!PRVM_clientfunction(CSQC_InputEvent))
+               r = false;
+       else
+       {
+               PRVM_clientglobalfloat(time) = cl.time;
+               PRVM_clientglobaledict(self) = cl.csqc_server2csqcentitynumber[cl.playerentity];
+               PRVM_G_FLOAT(OFS_PARM0) = eventtype;
+               PRVM_G_FLOAT(OFS_PARM1) = x; // key or x
+               PRVM_G_FLOAT(OFS_PARM2) = y; // ascii or y
+               prog->ExecuteProgram(prog, PRVM_clientfunction(CSQC_InputEvent), "QC function CSQC_InputEvent is missing");
+               r = CSQC_RETURNVAL != 0;
+       }
+CSQC_END
        return r;
 }
 
@@ -473,7 +473,6 @@ qboolean CL_VM_UpdateView (double frametime)
                return false;
        R_TimeReport("pre-UpdateView");
        CSQC_BEGIN
-               r_refdef.view.ismain = true;
                csqc_original_r_refdef_view = r_refdef.view;
                csqc_main_r_refdef_view = r_refdef.view;
                //VectorCopy(cl.viewangles, oldangles);
@@ -484,6 +483,10 @@ qboolean CL_VM_UpdateView (double frametime)
                // CSQC_UpdateView function does not call R_ClearScene as it should
                r_refdef.scene.numentities = 0;
                r_refdef.scene.numlights = 0;
+               // polygonbegin without draw2d arg has to guess
+               prog->polygonbegin_guess2d = false;
+               // free memory for resources that are no longer referenced
+               PRVM_GarbageCollection(prog);
                // pass in width and height as parameters (EXT_CSQC_1)
                PRVM_G_FLOAT(OFS_PARM0) = vid.width;
                PRVM_G_FLOAT(OFS_PARM1) = vid.height;
@@ -499,7 +502,7 @@ qboolean CL_VM_UpdateView (double frametime)
        return true;
 }
 
-qboolean CL_VM_ConsoleCommand (const char *cmd)
+qboolean CL_VM_ConsoleCommand (const char *text)
 {
        prvm_prog_t *prog = CLVM_prog;
        int restorevm_tempstringsbuf_cursize;
@@ -512,7 +515,7 @@ qboolean CL_VM_ConsoleCommand (const char *cmd)
                PRVM_clientglobalfloat(time) = cl.time;
                PRVM_clientglobaledict(self) = cl.csqc_server2csqcentitynumber[cl.playerentity];
                restorevm_tempstringsbuf_cursize = prog->tempstringsbuf.cursize;
-               PRVM_G_INT(OFS_PARM0) = PRVM_SetTempString(prog, cmd);
+               PRVM_G_INT(OFS_PARM0) = PRVM_SetTempString(prog, text);
                prog->ExecuteProgram(prog, PRVM_clientfunction(CSQC_ConsoleCommand), "QC function CSQC_ConsoleCommand is missing");
                prog->tempstringsbuf.cursize = restorevm_tempstringsbuf_cursize;
                r = CSQC_RETURNVAL != 0;
@@ -559,12 +562,10 @@ void CL_VM_Parse_StuffCmd (const char *msg)
                // temporarily so that it can be set by the cvar command,
                // and then reprotect it afterwards
                int crcflags = csqc_progcrc.flags;
-               int sizeflags = csqc_progcrc.flags;
                csqc_progcrc.flags &= ~CVAR_READONLY;
                csqc_progsize.flags &= ~CVAR_READONLY;
-               Cmd_ExecuteString (msg, src_command, true);
-               csqc_progcrc.flags = crcflags;
-               csqc_progsize.flags = sizeflags;
+               Cmd_ExecuteString(&cmd_client, msg, src_command, true);
+               csqc_progcrc.flags = csqc_progsize.flags = crcflags;
                return;
        }
 
@@ -595,7 +596,7 @@ void CL_VM_Parse_StuffCmd (const char *msg)
                                l = sizeof(buf) - 1;
                        strlcpy(buf, p, l + 1); // strlcpy needs a + 1 as it includes the newline!
 
-                       Cmd_ExecuteString(buf, src_command, true);
+                       Cmd_ExecuteString(&cmd_client, buf, src_command, true);
 
                        p += l;
                        if(*p == '\n')
@@ -603,13 +604,13 @@ void CL_VM_Parse_StuffCmd (const char *msg)
                        else
                                break; // end of string or overflow
                }
-               Cmd_ExecuteString("curl --clear_autodownload", src_command, true); // don't inhibit CSQC loading
+               Cmd_ExecuteString(&cmd_client, "curl --clear_autodownload", src_command, true); // don't inhibit CSQC loading
                return;
        }
 
        if(!cl.csqc_loaded)
        {
-               Cbuf_AddText(msg);
+               Cbuf_AddText(&cmd_client, msg);
                return;
        }
        CSQC_BEGIN
@@ -623,7 +624,7 @@ void CL_VM_Parse_StuffCmd (const char *msg)
                prog->tempstringsbuf.cursize = restorevm_tempstringsbuf_cursize;
        }
        else
-               Cbuf_AddText(msg);
+               Cbuf_AddText(&cmd_client, msg);
        CSQC_END
 }
 
@@ -717,7 +718,7 @@ void CL_VM_UpdateShowingScoresState (int showingscores)
                CSQC_END
        }
 }
-qboolean CL_VM_Event_Sound(int sound_num, float volume, int channel, float attenuation, int ent, vec3_t pos, int flags, float speed)
+qboolean CL_VM_Event_Sound(int sound_num, float fvolume, int channel, float attenuation, int ent, vec3_t pos, int flags, float speed)
 {
        prvm_prog_t *prog = CLVM_prog;
        qboolean r = false;
@@ -731,7 +732,7 @@ qboolean CL_VM_Event_Sound(int sound_num, float volume, int channel, float atten
                        PRVM_G_FLOAT(OFS_PARM0) = ent;
                        PRVM_G_FLOAT(OFS_PARM1) = CHAN_ENGINE2USER(channel);
                        PRVM_G_INT(OFS_PARM2) = PRVM_SetTempString(prog, cl.sound_name[sound_num] );
-                       PRVM_G_FLOAT(OFS_PARM3) = volume;
+                       PRVM_G_FLOAT(OFS_PARM3) = fvolume;
                        PRVM_G_FLOAT(OFS_PARM4) = attenuation;
                        VectorCopy(pos, PRVM_G_VECTOR(OFS_PARM5) );
                        PRVM_G_FLOAT(OFS_PARM6) = speed * 100.0f;
@@ -827,7 +828,7 @@ void CSQC_ReadEntities (void)
                                }
                                else
                                {
-                                       // LordHavoc: removing an entity that is already gone on
+                                       // LadyHavoc: removing an entity that is already gone on
                                        // the csqc side is possible for legitimate reasons (such
                                        // as a repeat of the remove message), so no warning is
                                        // needed
@@ -881,7 +882,7 @@ static void CLVM_end_increase_edicts(prvm_prog_t *prog)
 
        // link every entity except world
        for (i = 1, ent = prog->edicts;i < prog->num_edicts;i++, ent++)
-               if (!ent->priv.server->free)
+               if (!ent->priv.server->free && !VectorCompare(PRVM_clientedictvector(ent, absmin), PRVM_clientedictvector(ent, absmax)))
                        CL_LinkEdict(ent);
 }
 
@@ -1014,7 +1015,7 @@ void CL_VM_Init (void)
                }
                else
                {
-                       Con_DPrintf("Not using buffered \"%s\" (buffered: %p, %d)\n", csprogsfn, cls.caughtcsprogsdata, (int) cls.caughtcsprogsdatasize);
+                       Con_DPrintf("Not using buffered \"%s\" (buffered: %p, %d)\n", csprogsfn, (void *)cls.caughtcsprogsdata, (int) cls.caughtcsprogsdatasize);
                        csprogsdata = FS_LoadFile(csprogsfn, tempmempool, true, &csprogsdatasize);
                }
        }
@@ -1030,7 +1031,7 @@ void CL_VM_Init (void)
                {
                        if (cls.demoplayback)
                        {
-                               Con_Printf("^1Warning: Your %s is not the same version as the demo was recorded with (CRC/size are %i/%i but should be %i/%i)\n", csqc_progname.string, csprogsdatacrc, (int)csprogsdatasize, requiredcrc, requiredsize);
+                               Con_Printf(CON_WARN "Warning: Your %s is not the same version as the demo was recorded with (CRC/size are %i/%i but should be %i/%i)\n", csqc_progname.string, csprogsdatacrc, (int)csprogsdatasize, requiredcrc, requiredsize);
                                // Mem_Free(csprogsdata);
                                // return;
                                // We WANT to continue here, and play the demo with different csprogs!
@@ -1039,7 +1040,7 @@ void CL_VM_Init (void)
                        else
                        {
                                Mem_Free(csprogsdata);
-                               Con_Printf("^1Your %s is not the same version as the server (CRC is %i/%i but should be %i/%i)\n", csqc_progname.string, csprogsdatacrc, (int)csprogsdatasize, requiredcrc, requiredsize);
+                               Con_Printf(CON_ERROR "Your %s is not the same version as the server (CRC is %i/%i but should be %i/%i)\n", csqc_progname.string, csprogsdatacrc, (int)csprogsdatasize, requiredcrc, requiredsize);
                                CL_Disconnect();
                                return;
                        }
@@ -1050,15 +1051,15 @@ void CL_VM_Init (void)
                if (requiredcrc >= 0)
                {
                        if (cls.demoplayback)
-                               Con_Printf("CL_VM_Init: demo requires CSQC, but \"%s\" wasn't found\n", csqc_progname.string);
+                               Con_Printf(CON_ERROR "CL_VM_Init: demo requires CSQC, but \"%s\" wasn't found\n", csqc_progname.string);
                        else
-                               Con_Printf("CL_VM_Init: server requires CSQC, but \"%s\" wasn't found\n", csqc_progname.string);
+                               Con_Printf(CON_ERROR "CL_VM_Init: server requires CSQC, but \"%s\" wasn't found\n", csqc_progname.string);
                        CL_Disconnect();
                }
                return;
        }
 
-       PRVM_Prog_Init(prog);
+       PRVM_Prog_Init(prog, &cmd_client);
 
        // allocate the mempools
        prog->progs_mempool = Mem_AllocPool(csqc_progname.string, 0, NULL);
@@ -1090,7 +1091,7 @@ void CL_VM_Init (void)
 
        if (!prog->loaded)
        {
-               Host_Error("CSQC %s ^2failed to load\n", csprogsfn);
+               Host_Error("CSQC %s failed to load\n", csprogsfn);
                if(!sv.active)
                        CL_Disconnect();
                Mem_Free(csprogsdata);
@@ -1145,6 +1146,9 @@ void CL_VM_Init (void)
        // call the prog init
        prog->ExecuteProgram(prog, PRVM_clientfunction(CSQC_Init), "QC function CSQC_Init is missing");
 
+       // Once CSQC_Init was called, we consider csqc code fully initialized.
+       prog->inittime = host.realtime;
+
        cl.csqc_loaded = true;
 
        cl.csqc_vidvars.drawcrosshair = false;
@@ -1157,21 +1161,21 @@ void CL_VM_Init (void)
 void CL_VM_ShutDown (void)
 {
        prvm_prog_t *prog = CLVM_prog;
-       Cmd_ClearCsqcFuncs();
+       Cmd_ClearCSQCCommands(&cmd_client);
        //Cvar_SetValueQuick(&csqc_progcrc, -1);
        //Cvar_SetValueQuick(&csqc_progsize, -1);
        if(!cl.csqc_loaded)
                return;
-       CSQC_BEGIN
-               if (prog->loaded)
-               {
-                       PRVM_clientglobalfloat(time) = cl.time;
-                       PRVM_clientglobaledict(self) = 0;
-                       if (PRVM_clientfunction(CSQC_Shutdown))
-                               prog->ExecuteProgram(prog, PRVM_clientfunction(CSQC_Shutdown), "QC function CSQC_Shutdown is missing");
-               }
-               PRVM_Prog_Reset(prog);
-       CSQC_END
+CSQC_BEGIN
+       if (prog->loaded)
+       {
+               PRVM_clientglobalfloat(time) = cl.time;
+               PRVM_clientglobaledict(self) = 0;
+               if (PRVM_clientfunction(CSQC_Shutdown))
+                       prog->ExecuteProgram(prog, PRVM_clientfunction(CSQC_Shutdown), "QC function CSQC_Shutdown is missing");
+       }
+       PRVM_Prog_Reset(prog);
+CSQC_END
        Con_DPrint("CSQC ^1unloaded\n");
        cl.csqc_loaded = false;
 }
@@ -1186,15 +1190,13 @@ qboolean CL_VM_GetEntitySoundOrigin(int entnum, vec3_t out)
 
        CSQC_BEGIN;
 
-       // FIXME consider attachments here!
-
        ed = PRVM_EDICT_NUM(entnum - MAX_EDICTS);
 
        if(!ed->priv.required->free)
        {
                mod = CL_GetModelFromEdict(ed);
                VectorCopy(PRVM_clientedictvector(ed, origin), out);
-               if(CL_GetTagMatrix(prog, &matrix, ed, 0) == 0)
+               if(CL_GetTagMatrix(prog, &matrix, ed, 0, NULL) == 0)
                        Matrix4x4_OriginFromMatrix(&matrix, out);
                if (mod && mod->soundfromcenter)
                        VectorMAMAM(1.0f, out, 0.5f, mod->normalmins, 0.5f, mod->normalmaxs, out);